package it.amattioli.guidate.browsing;

import static it.amattioli.dominate.validation.ValidationResult.ResultType.INVALID;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ResourceBundle;

import org.apache.commons.beanutils.NestedNullException;
import org.apache.commons.beanutils.PropertyUtils;
import org.zkoss.util.Locales;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.WrongValueException;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.Events;
import org.zkoss.zk.ui.event.OpenEvent;
import org.zkoss.zul.Bandbox;

import it.amattioli.applicate.browsing.EntitySelector;
import it.amattioli.applicate.browsing.EntitySelector.Match;
import it.amattioli.dominate.validation.ValidationResult;
import it.amattioli.guidate.containers.BackBeans;
import it.amattioli.guidate.properties.PropertyNameRetriever;
import it.amattioli.guidate.properties.TextPropertyComposer;
import it.amattioli.guidate.validators.ValidatingComposer;

public class EntitySelectorComposer extends TextPropertyComposer {
	private static final String IS_OPEN_ATTRIBUTE = "it.amattioli.locate.ui.zk.PlaceSelectorComposer.isOpen";
	
	public boolean isOpen(Component comp) {
		return (Boolean)comp.getAttribute(IS_OPEN_ATTRIBUTE);
	}
	
	public void setOpen(Component comp, boolean open) {
		comp.setAttribute(IS_OPEN_ATTRIBUTE, open);
	}
	
	public void registerAsValidable(Component comp) throws Exception {
    	if (comp.getParent() != null) {
        	Component container = BackBeans.findContainer(comp.getParent());
    		if (container != null) {
    			Events.sendEvent(new Event(ValidatingComposer.ON_REGISTER_VALIDABLE_ELEMENT, container, comp));
    		}
    	}
    }
	
	public void onCreate(Event evt) throws Exception {
		Component comp = evt.getTarget();
		registerAsValidable(comp);
		setOpen(comp, false);
		Object val = null;
		try {
			Object backBean = getBackBean(comp);
			if (backBean != null) {
				String selectorName = (String)comp.getAttribute("selectorName");
				val = PropertyUtils.getProperty(backBean, selectorName);
			}
		} catch (NestedNullException e) {
			e.printStackTrace();
			val = null;
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
			val = null;
		}
		if (val != null) {
			comp.setAttribute(BackBeans.BACK_BEAN_ATTRIBUTE, val);
			comp.setAttribute(PropertyNameRetriever.PROPERTY_NAME, "searchValue");
			setPropertyClass(String.class);
			setPropertyName("searchValue");
			EntitySelector<?, ?> selector = (EntitySelector<?, ?>)val;
			final Bandbox bandbox = (Bandbox)comp;
			bandbox.getDropdown().setAttribute(BackBeans.BACK_BEAN_ATTRIBUTE, selector.getEntityBrowser());
			selector.addPropertyChangeListener(new PropertyChangeListener() {
	
				@Override
				public void propertyChange(PropertyChangeEvent evt) {
					if ("selectedObject".equals(evt.getPropertyName()) && isOpen(bandbox)) {
						bandbox.close();
					}
				}
				
			});
		}
		super.onCreate(evt);
	}
	
	public void onValidateElement(Event evt) {
		EntitySelector<?, ?> selector = (EntitySelector<?, ?>)evt.getTarget().getAttribute(BackBeans.BACK_BEAN_ATTRIBUTE);
		if (selector != null) {
			if (Match.NONE.equals(selector.getMatch())) {
				String msg = ResourceBundle.getBundle("it/amattioli/guidate/browsing/EntitySelectorMessages", Locales.getCurrent()).getString("entitySelector.MatchNone");
				throw new WrongValueException(evt.getTarget(), msg);
			}
			ValidationResult result = selector.validate();
			if (result != null && INVALID.equals(result.getType())) {
				throw new WrongValueException(evt.getTarget(), result.getMessage());
			}
		}
    }
	
	@Override
	public void onBlur(Event evt) throws Exception {
		super.onBlur(evt);
		openBandboxIfMultipleMatches((Bandbox)evt.getTarget());
		Events.postEvent("onValidateElement", evt.getTarget(), null);
	}

	private void openBandboxIfMultipleMatches(Bandbox bandbox) {
		EntitySelector<?, ?> selector = (EntitySelector<?, ?>)bandbox.getAttribute(BackBeans.BACK_BEAN_ATTRIBUTE);
		if (Match.MULTIPLE.equals(selector.getMatch())) {
			bandbox.open();
		}
	}

	public void onOpen(OpenEvent evt) throws Exception {
		setOpen(evt.getTarget(), evt.isOpen());
		Events.sendEvent(new Event("onOpenBandpopup", ((Bandbox)evt.getTarget()).getDropdown()));
	}
}
