package it.amattioli.guidate.browsing.calendar;

import it.amattioli.encapsulate.browsers.CalendarBrowser;
import it.amattioli.guidate.containers.BackBeans;
import it.amattioli.guidate.properties.PropertyValuesComposer;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zk.ui.event.Events;
import org.zkoss.zul.Div;
import org.zkoss.zul.Hbox;
import org.zkoss.zul.Label;
import org.zkoss.zul.Listbox;
import org.zkoss.zul.Toolbarbutton;
import org.zkoss.zul.Vbox;

public class Navigator extends Vbox {
	private CalendarBrowser<?, ?> browser;
	private Label intervalName;
	private Listbox intervalType;
	private PropertyChangeListener intervalChangeListener;
	
	public void onCreate(Event evt) throws Exception {
		this.browser = (CalendarBrowser<?, ?>)BackBeans.findBackBean(evt.getTarget());
		this.setAttribute("backBean", browser);
		
		intervalType = new Listbox();
		intervalType.setAttribute("propertyName", "intervalType");
		intervalType.setRows(1);
		intervalType.setMold("select");
		PropertyValuesComposer intervalTypeComposer = new PropertyValuesComposer();
		intervalTypeComposer.doAfterCompose(intervalType);
		intervalType.setParent(this);
		
		Hbox navigator = new Hbox();
		navigator.setWidth("100%");
		navigator.setWidths("10%,80%,10%");

		createPrevButton(navigator);
		
		createIntervalName(navigator);
		
		createNextButton(navigator);
		
		navigator.setParent(this);
		
		Events.sendEvent(new Event(Events.ON_CREATE, intervalType));
	}

	private void createNextButton(Hbox container) {
		Div nextButtonContainer = new Div();
		nextButtonContainer.setAlign("right");
		Toolbarbutton nextButton = new Toolbarbutton();
		nextButton.setImage("zkau/web/guidate/browsing/calnext.gif");
		nextButton.addEventListener("onClick", new EventListener() {

			@Override
			public void onEvent(Event evt) throws Exception {
				browser.nextInterval();
			}
			
		});
		nextButton.setParent(nextButtonContainer);
		nextButtonContainer.setParent(container);
	}

	private void createIntervalName(Hbox container) {
		Div intervalNameContainer = new Div();
		intervalNameContainer.setAlign("center");
		intervalName = new Label();
		intervalName.setValue(browser.getInterval().toString());
		intervalChangeListener = new PropertyChangeListener() {

			public void propertyChange(PropertyChangeEvent evt) {
				if ("interval".equals(evt.getPropertyName())) {
					intervalName.setValue(browser.getInterval().toString());
				}
			}
			
		};
		browser.addPropertyChangeListener(intervalChangeListener);
		intervalName.setParent(intervalNameContainer);
		intervalNameContainer.setParent(container);
	}

	private void createPrevButton(Hbox container) {
		Div prevButtonContainer = new Div();
		prevButtonContainer.setAlign("left");
		Toolbarbutton prevButton = new Toolbarbutton();
		prevButton.setImage("zkau/web/guidate/browsing/calprec.gif");
		prevButton.addEventListener("onClick", new EventListener() {

			@Override
			public void onEvent(Event evt) throws Exception {
				browser.previousInterval();
			}
			
		});
		prevButton.setParent(prevButtonContainer);
		prevButtonContainer.setParent(container);
	}

}
